@@ -3,7 +3,7 @@ pg_activity
 author: Julien Tachoires <julmon@gmail.com>
 license: PostgreSQL License
 
-Copyright (c) 2012 - 2014, Julien Tachoires
+Copyright (c) 2012 - 2015, Julien Tachoires
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose, without fee, and without a written
@@ -134,7 +134,8 @@ class Data:
         port = 5432,
         user = 'postgres',
         password = None,
-        database = 'postgres'):
+        database = 'postgres',
+        rds_mode = False):
         """
         Connect to a PostgreSQL server and return
         cursor & connector.
@@ -150,7 +151,7 @@ class Data:
                     password = str(password),
                     connection_factory = psycopg2.extras.DictConnection
                     )
-            except psycopg2.Error, psy_err:
+            except psycopg2.Error as psy_err:
                 if host is None:
                     raise psy_err
         if self.pg_conn is None: # fallback on TCP/IP connection
@@ -163,27 +164,28 @@ class Data:
                 connection_factory = psycopg2.extras.DictConnection
                 )
         self.pg_conn.set_isolation_level(0)
-        cur = self.pg_conn.cursor()
-        cur.execute("SELECT current_setting('is_superuser')")
-        ret = cur.fetchone()
-        if ret[0] != "on":
-            raise Exception("Must be run with database superuser privileges.")
+        if rds_mode != True: # Make sure we are using superuser if not on RDS
+          cur = self.pg_conn.cursor()
+          cur.execute("SELECT current_setting('is_superuser')")
+          ret = cur.fetchone()
+          if ret[0] != "on":
+              raise Exception("Must be run with database superuser privileges.")
 
     def pg_is_local_access(self,):
         """
         Verify if the user running pg_activity can acces 
-        system informations for a postgres process.
+        system informations for the postmaster process.
         """
-        for psproc in psutil.process_iter():
-            try:
-                name = psproc.name()
-            except NameError:
-                name = psproc.name
-            except TypeError:
-                name = psproc.name
-            if name in ('postgres', 'postmaster', 'edb-postgres'):
+        try:
+            query = "SELECT setting||'/postmaster.pid' AS pid_file FROM pg_settings WHERE name = 'data_directory'"
+            cur = self.pg_conn.cursor()
+            cur.execute(query)
+            ret = cur.fetchone()
+            pid_file = ret['pid_file']
+            with open(pid_file, 'r') as fd:
+                pid = fd.readlines()[0].strip()
                 try:
-                    proc = PSProcess(psproc.pid)
+                    proc = PSProcess(int(pid))
                     proc.io_counters()
                     proc.cpu_times()
                     return True
@@ -191,7 +193,8 @@ class Data:
                     return False
                 except Exception:
                     return False
-        return False
+        except Exception:
+            return False
 
     def pg_get_version(self,):
         """
@@ -243,7 +246,7 @@ class Data:
         from a string (SELECT version()). 
         """
         res = re.match(
-            r"^(PostgreSQL|EnterpriseDB) ([0-9]+)\.([0-9]+)(devel|beta[0-9]+)",
+            r"^(PostgreSQL|EnterpriseDB) ([0-9]+)\.([0-9]+)(devel|beta[0-9]+|rc[0-9]+)",
             text_version)
         if res is not None:
             rmatch = res.group(2)
@@ -256,7 +259,7 @@ class Data:
             return
         raise Exception('Undefined PostgreSQL version.')
 
-    def pg_get_db_info(self, prev_db_infos):
+    def pg_get_db_info(self, prev_db_infos, using_rds = False):
         """
         Get current sum of transactions, total size and  timestamp.
         """
@@ -269,6 +272,7 @@ class Data:
     FROM
         pg_database
         """
+        query += "\nWHERE datname <> 'rdsadmin'" if using_rds else ''
         cur = self.pg_conn.cursor()
         cur.execute(query,)
         ret = cur.fetchone()
