@@ -3,7 +3,7 @@ pg_activity
 author: Julien Tachoires <julmon@gmail.com>
 license: PostgreSQL License
 
-Copyright (c) 2012 - 2014, Julien Tachoires
+Copyright (c) 2012 - 2015, Julien Tachoires
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose, without fee, and without a written
@@ -22,6 +22,8 @@ BASIS, AND JULIEN TACHOIRES HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE,
 SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 """
 
+from __future__ import print_function
+
 import curses
 import re
 import time
@@ -29,6 +31,7 @@ import sys
 from datetime import timedelta
 from pgactivity.Data import Data
 import psutil
+from getpass import getpass
 
 # Define some color pairs
 C_BLACK_GREEN = 1
@@ -323,6 +326,54 @@ class UI:
         # Default filesystem blocksize
         self.fs_blocksize = 4096
         # Init curses
+        # self.__init_curses()
+
+    def set_is_local(self, is_local):
+        """
+        Set self.is_local
+        """
+        self.is_local = is_local
+
+    def get_is_local(self,):
+        """
+        Get self.is_local
+        """
+        return self.is_local
+
+    def get_mode(self,):
+        """
+        Get self.mode
+        """
+        return self.mode
+
+    def set_start_line(self, start_line):
+        """
+        Set self.start_line
+        """
+        self.start_line = start_line
+
+    def set_buffer(self, uibuffer):
+        """
+        Set self.uibuffer
+        """
+        self.uibuffer = uibuffer
+
+    def set_blocksize(self, blocksize):
+        """
+        Set blocksize
+        """
+        if not isinstance(blocksize, int):
+            raise Exception('Unvalid blocksize value.')
+        if blocksize != 0 and not ((blocksize & (blocksize - 1)) == 0):
+            raise Exception('Unvalid blocksize value.')
+        if not blocksize > 0:
+            raise Exception('Unvalid blocksize value.')
+        self.fs_blocksize = int(blocksize)
+
+    def init_curses(self,):
+        """
+        Initialize curses environment and colors.
+        """
         self.__init_curses()
         # Columns colors definition
         self.line_colors = {
@@ -418,48 +469,6 @@ class UI:
             }
         }
 
-    def set_is_local(self, is_local):
-        """
-        Set self.is_local
-        """
-        self.is_local = is_local
-
-    def get_is_local(self,):
-        """
-        Get self.is_local
-        """
-        return self.is_local
-
-    def get_mode(self,):
-        """
-        Get self.mode
-        """
-        return self.mode
-
-    def set_start_line(self, start_line):
-        """
-        Set self.start_line
-        """
-        self.start_line = start_line
-
-    def set_buffer(self, uibuffer):
-        """
-        Set self.uibuffer
-        """
-        self.uibuffer = uibuffer
-
-    def set_blocksize(self, blocksize):
-        """
-        Set blocksize
-        """
-        if not isinstance(blocksize, int):
-            raise Exception('Unvalid blocksize value.')
-        if blocksize != 0 and not ((blocksize & (blocksize - 1)) == 0):
-            raise Exception('Unvalid blocksize value.')
-        if not blocksize > 0:
-            raise Exception('Unvalid blocksize value.')
-        self.fs_blocksize = int(blocksize)
-
     def __init_curses(self,):
         """
         Initialize curses environment.
@@ -559,6 +568,9 @@ class UI:
             self.win.erase()
         except KeyboardInterrupt:
             pass
+        except AttributeError:
+            # Curses not initialized yet
+            return
         curses.nocbreak()
         curses.echo()
         try:
@@ -572,8 +584,8 @@ class UI:
         Function called on a process kill.
         """
         self.at_exit_curses()
-        print "FATAL: Killed with signal %s ." % (str(signal),)
-        print "%s" % (str(frame),)
+        print("FATAL: Killed with signal %s ." % (str(signal),))
+        print("%s" % (str(frame),))
         sys.exit(1)
 
     def set_nocolor(self,):
@@ -624,6 +636,13 @@ class UI:
         msg = re.sub(r"\s$", r"", msg)
         return msg
 
+    def ask_password(self, ):
+        """
+        Ask for PostgreSQL user password
+        """
+        password = getpass()
+        return password
+
     def check_window_size(self,):
         """
         Update window's size
@@ -654,7 +673,7 @@ class UI:
         while 1:
             try:
                 k = self.win.getch()
-            except KeyboardInterrupt, err:
+            except KeyboardInterrupt as err:
                 raise err
             if k == ord('q'):
                 curses.endwin()
@@ -664,7 +683,7 @@ class UI:
                 return 0
 
             if k == curses.KEY_RESIZE:
-                if self.uibuffer is not None and self.uibuffer.has_key('procs'):
+                if self.uibuffer is not None and 'procs' in self.uibuffer:
                     self.check_window_size()
                     self.refresh_window(
                         self.uibuffer['procs'],
@@ -824,7 +843,7 @@ class UI:
         """
         Ask for terminating some backends
         """
-        if len(pids) == 1: 
+        if len(pids) == 1:
             colno = self.__print_string(
                         (self.maxy - 1),
                         0,
@@ -856,7 +875,7 @@ class UI:
         while 1:
             try:
                 key = self.win.getch()
-            except KeyboardInterrupt, err:
+            except KeyboardInterrupt as err:
                 raise err
             # quit
             if key == ord('q'):
@@ -900,7 +919,7 @@ class UI:
         old_verbose_mode = self.verbose_mode
         self.verbose_mode = PGTOP_TRUNCATE
 
-        # Refresh lines with this verbose mode 
+        # Refresh lines with this verbose mode
         self.__scroll_window(process, flag, indent, 0)
 
         self.__help_key_interactive()
@@ -920,7 +939,7 @@ class UI:
             known = False
             try:
                 k = self.win.getch()
-            except KeyboardInterrupt, err:
+            except KeyboardInterrupt as err:
                 raise err
             if k == -1:
                 nb_nk += 1
@@ -1047,7 +1066,7 @@ class UI:
         do_refresh = False
         try:
             key = self.win.getch()
-        except KeyboardInterrupt, err:
+        except KeyboardInterrupt as err:
             raise err
         if key == ord('q'):
             curses.endwin()
@@ -1112,7 +1131,7 @@ class UI:
         # Refresh
         if key == ord('R'):
             known = True
-        
+
         if key == ord('u'):
             self.__empty_pid_yank()
             known = True
@@ -1123,22 +1142,23 @@ class UI:
 
         if key == curses.KEY_RESIZE and \
             self.uibuffer is not None and \
-            self.uibuffer.has_key('procs'):
+            'procs' in self.uibuffer:
             do_refresh = True
 
         if do_refresh is True and \
             self.uibuffer is not None and \
-            self.uibuffer.has_key('procs'):
-            self.check_window_size()
-            self.refresh_window(
-                self.uibuffer['procs'],
-                self.uibuffer['extras'],
-                self.uibuffer['flag'],
-                self.uibuffer['indent'],
-                self.uibuffer['io'],
-                self.uibuffer['tps'],
-                self.uibuffer['size_ev'],
-                self.uibuffer['total_size'])
+            type(self.uibuffer) is dict and \
+            'procs' in self.uibuffer:
+                self.check_window_size()
+                self.refresh_window(
+                    self.uibuffer['procs'],
+                    self.uibuffer['extras'],
+                    self.uibuffer['flag'],
+                    self.uibuffer['indent'],
+                    self.uibuffer['io'],
+                    self.uibuffer['tps'],
+                    self.uibuffer['size_ev'],
+                    self.uibuffer['total_size'])
 
         curses.flushinp()
         t_end = time.time()
@@ -1166,7 +1186,7 @@ class UI:
             write_count_delta = 0
             for pid, new_proc in new_procs.items():
                 try:
-                    if process.has_key(pid):
+                    if pid in process:
                         n_io_time = time.time()
                         # Getting informations from the previous loop
                         proc = process[pid]
@@ -1194,7 +1214,7 @@ class UI:
                         proc.set_extra(
                             'io_time',
                             n_io_time)
- 
+
                         # Global io counters
                         read_bytes_delta  += proc.get_extra('read_delta')
                         write_bytes_delta += proc.get_extra('write_delta')
@@ -1207,11 +1227,11 @@ class UI:
 
                     proc.set_extra(
                         'mem_percent',
-                        proc.get_extra('psutil_proc').get_memory_percent())
+                        proc.get_extra('psutil_proc').memory_percent())
                     proc.set_extra(
                         'cpu_percent',
                         proc.get_extra('psutil_proc').\
-                            get_cpu_percent(interval=0))
+                            cpu_percent(interval=0))
                     new_procs[pid] = proc
                     procs.append({
                         'pid': pid,
@@ -1232,7 +1252,7 @@ class UI:
                     pass
                 except psutil.AccessDenied:
                     pass
-                except Exception, err:
+                except Exception as err:
                     raise err
             # store io counters
             if read_bytes_delta > 0:
@@ -1314,7 +1334,7 @@ class UI:
 
         try:
             k = self.win.getch()
-        except KeyboardInterrupt, err:
+        except KeyboardInterrupt as err:
             raise err
         if k == ord('q'):
             curses.endwin()
@@ -1378,11 +1398,11 @@ class UI:
             known = True
 
         if k == curses.KEY_RESIZE and self.uibuffer is not None and \
-            self.uibuffer.has_key('procs'):
+            'procs' in self.uibuffer:
             do_refresh = True
 
         if do_refresh is True and self.uibuffer is not None and \
-            self.uibuffer.has_key('procs'):
+            'procs' in self.uibuffer:
             self.check_window_size()
             self.refresh_window(
                 self.uibuffer['procs'],
@@ -1496,7 +1516,7 @@ class UI:
         self.lineno += 1
 
     def __print_header(self, pg_version, hostname, user, host, \
-        port, ios, tps, size_ev, total_size):
+        port, database, ios, tps, size_ev, total_size):
         """
         Print window header
         """
@@ -1543,6 +1563,15 @@ class UI:
                     colno,
                     port,
                     self.__get_color(C_CYAN))
+        colno += self.__print_string(
+                    self.lineno,
+                    colno,
+                    "/")
+        colno += self.__print_string(
+                    self.lineno,
+                    colno,
+                    database,
+                    self.__get_color(C_CYAN))
         colno += self.__print_string(
                     self.lineno,
                     colno,
@@ -1617,7 +1646,7 @@ class UI:
         """
         self.win.erase()
         self.lineno = 0
-        text = "pg_activity %s - (c) 2012-2014 Julien Tachoires" % \
+        text = "pg_activity %s - (c) 2012-2015 Julien Tachoires" % \
             (self.version)
         self.__print_string(
                 self.lineno,
@@ -1734,7 +1763,7 @@ class UI:
         self.win.timeout(-1)
         try:
             self.win.getch()
-        except KeyboardInterrupt, err:
+        except KeyboardInterrupt as err:
             raise err
 
     def __display_help_key(self, lineno, colno, key, help_msg):
@@ -1759,7 +1788,7 @@ class UI:
         """
 
         self.lines = []
-        (pg_version, hostname, user, host, port) = extras
+        (pg_version, hostname, user, host, port, dbname) = extras
         self.win.erase()
         self.__print_header(
             pg_version,
@@ -1767,6 +1796,7 @@ class UI:
             user,
             host,
             port,
+            dbname,
             ios,
             tps,
             size_ev,
@@ -1892,11 +1922,14 @@ class UI:
                                 self.line_colors['mode_yellow'][typecolor])
 
         if flag & PGTOP_FLAG_TIME:
-            if process['duration'] >= 1:
+            if process['duration'] >= 1 and process['duration'] < 60000:
                 ctime = timedelta(seconds=float(process['duration']))
                 mic = '%.6d' % (ctime.microseconds)
                 ctime = "%s:%s.%s" % (str((ctime.seconds // 60)).zfill(2), \
                             str((ctime.seconds % 60)).zfill(2), str(mic)[:2])
+            elif process['duration'] >= 60000:
+                ctime = "%s h" % str(int(process['duration'] / 3600))
+
             if process['duration'] < 1:
                 colno += self.__print_string(
                             l_lineno,
